// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const Signals = imports.signals;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const REG_GET = "/bin/regget";
const REG_SET = "/bin/regset";
const SECTION = "System";

const WyseDomainList = new Lang.Class({
    Name: 'WyseDomainList',

    _init: function() {
        //log("WyseDomainList: _init()");

        this.domain_enabled = false;
        this._lastLoggedInUser = "";
        this._useLastDomainName = false;
        this._domains = new Array();
        this._activeDomain = "";
        this._loadDomainsFromConfig();
    },

    _pushDomain: function(domain) {
        let d = domain.replace(/\s+/g, '');

        if (d.length > 0) {
            if (this._domains.indexOf(d) == -1)
                this._domains.push(d);
        }
    },

    _createDomainMenu: function() {
        this._menu.removeAll();
        //log("Domain list size: " + this._domains.length);

        for (let i = 0; i < this._domains.length; i++) {
            let item = new PopupMenu.PopupMenuItem(this._domains[i]);
            this._menu.addMenuItem(item);

            item.connect('activate', Lang.bind(this, function(item) {
                                     //this.setActiveDomain(this._domains[i]);
                                     //log("selected: " + item.label.text);
                                     this._setActiveDomain(item.label.text);
                                     }));
        }
    },

    _setActiveDomain: function(domain) {
        this._activeDomain = domain;
        this.emit('domain-activated');
        this._domainEntry.set_text(domain);
    },

    _generate: function() {
        //log("WyseDomainList: _create()");
        this.actor = new St.BoxLayout({ style: 'spacing: 8px;',
                                        vertical: true });

        this._domainLabel = new St.Label({ style_class: 'login-dialog-prompt-label',
                                           text: _("Domain: ") });

        this.actor.add(this._domainLabel,
                       { expand: true,
                         x_fill: true,
                         y_fill: true,
                         x_align: St.Align.START });

        this._domainEntry = new St.Entry({ style_class: 'login-dialog-prompt-entry',
                                           can_focus: true });
        this.actor.add(this._domainEntry,
                                { expand: true,
                                  x_fill: true,
                                  y_fill: true,
                                  x_align: St.Align.START });
        let icon = new St.Icon( { icon_name: 'go-down-symbolic', icon_size: 16 } );

        //log("Active Domain: " + this._activeDomain);
        this._domainEntry.set_text(this._activeDomain);
        this._domainEntry.set_secondary_icon(icon);

        this._menu = new PopupMenu.PopupMenu(this._domainEntry, 0, St.Side.TOP);
        Main.uiGroup.add_actor(this._menu.actor);
        this._menu.actor.hide();

        this._menu.connect('open-state-changed',
                           Lang.bind(this, function(menu, isOpen) {
                                if (isOpen)
                                    this._domainEntry.add_style_pseudo_class('active');
                                else
                                    this._domainEntry.remove_style_pseudo_class('active');
                           }));

        this._manager = new PopupMenu.PopupMenuManager({ actor: this._domainEntry });
        this._manager.addMenu(this._menu);

        this._domainEntry.connect('secondary-icon-clicked', Lang.bind(this, function() {
                                                    //log("Clicked menu....");
                                                    this._menu.toggle();
                                                    }));

        this._domainEntry.connect('key-press-event',
                                  Lang.bind(this, function(actor, event) {
                                    let symbol = event.get_key_symbol();
                                    if ( symbol == Clutter.KEY_Down ) {
                                        //log("Down arrow key pressed");
                                        actor.emit('secondary-icon-clicked');
                                        this._menu.actor.navigate_focus(null, Gtk.DirectionType.DOWN, true);
                                        //return true;
                                    }
                                 }));

        this._createDomainMenu();
    },

    _loadDomainsFromConfig: function() {
        //log("_loadDomainsFromConfig");
        
        try {
            this.domain_enabled = !this.regGet('System', 'DisableDomain', 'False');

            if (this.domain_enabled) {
                //log("Domain enabled");

                let content = this.regGet('System', 'DomainList', "");

                if (!content) {
                    //log("Domains list not found in registry");
                    this.domain_enabled = false;
                    return;
                }

                let domains = content.split(';');
                //  Sort the domain list
                domains.sort();

                for (let i = 0; i < domains.length; i++) {
                    //log("Domain = " + domains[i]);
                    this._pushDomain(domains[i]);
                }

                let useLastLoginName = this.regGet('System', 'UseLastLoginName', 'False');
                if (useLastLoginName) {
                    this._lastLoggedInUser = this.regGet('System', 'LastLoggedInUser', '');
                    //log("LastLoginName: " + this._lastLoggedInUser);
                }
 
                let lastLoggedInDomain = "";
                this._useLastDomainName = this.regGet('System', 'UseLastDomainName', 'False');

                if (this._useLastDomainName) {
                    lastLoggedInDomain = this.regGet('System', 'LastLoggedInDomain', '');
                }
                //log("Last loggedin domain: " + lastLoggedInDomain);

                if (lastLoggedInDomain) {
                    this._activeDomain = lastLoggedInDomain;
                } else if (domains.length > 0) {
                    //log("Active Domain: " + domains[0]);
                    this._activeDomain = domains[0];

                    //log("Active Domain: " + this._activeDomain);
                }

                //  Add 'Local login' entry into the domain drop down list
                //  for user to choose it for local login.
                this._domains.push( _("Local login") );
            }
        } catch(e) {
            log("_loadDomainsFromConfig(): exception: " + e);
        }

        if (this.domain_enabled) {
            this._generate();
        }
    },

    isDomainEnabled: function() {
        return this.domain_enabled;
    }, 

    getActiveDomain: function() {
        return this._activeDomain;
    }, 

    getLastLoggedInUsername: function() {
        return this._lastLoggedInUser;
    },

    regGet: function(section, key, defaultValue) {
        let result = "";
        defaultValue = typeof defaultValue != 'undefined' ? defaultValue : "";

        try {
            let [res, out] = GLib.spawn_sync(null,
                                             [ REG_GET, section, key, defaultValue ],
                                             null,
                                             GLib.SpawnFlags.SEARCH_PATH,
                                             null);
            if (res) {
                result = out.toString().replace(/\n/g, "");
            }

            //  Handle boolean values
            if (result.match(/^yes$/i) ||
                result.match(/^true$/i) ||
                result.match(/^on$/i)) {
                result = true;
            } else if (result.match(/^no$/i) ||
                result.match(/^false$/i) ||
                result.match(/^off$/i)) {
                result = false;
            }

        } catch (e) {
            log("regGet(): " + e); 
        }
        return result;
    },

    regSet: function(section, key, value) {
        let result = false;

        try {
            let [res, out] = GLib.spawn_sync(null,
                                             [ REG_SET, section, key, value ],
                                             null,
                                             GLib.SpawnFlags.SEARCH_PATH,
                                             null);
            result = res;
        } catch (e) {
            log("regSet(): " + e); 
        }
        return result;
    },

});

Signals.addSignalMethods(WyseDomainList.prototype);

